const InternalContext = require('./core/InternalContext')
const HttpClient = require('./net/HttpClient')
const Config = require('./util/Config')
const instance = new InternalContext()

module.exports.startup = async function (runtime, source) {
    let config = new Config(runtime)
    if (typeof source === 'string') {
        if (HttpClient.isAbsoluteUrl(source)) {
            let response = await new HttpClient().get(source)
            if (response.status === 200) {
                config.source = JSON.parse(response.body)
            } else {
                throw new Error('failed download: ' + source)
            }
        } else {
            config.source = require(config.resolvePath(source))
        }
    } else {
        config.source = source
    }
    await instance.initialize(config)
    await instance.startup()
}

module.exports.shutdown = function () {
    instance.shutdown()
}

module.exports.scan = async function (path, filter) {
    await instance.scan(path, filter)
}

module.exports.installController = function (controller) {
    instance.installController(controller)
}

module.exports.installRepositories = async function (path) {
    await instance.installRepositories(path)
}

module.exports.installRepository = async function (context, repository) {
    await instance.installRepository(context, repository)
}

module.exports.repository = function (repository) {
    return repository
}

module.exports.schedule = function (task, isTransaction) {
    instance.schedule(task, isTransaction)
}

module.exports.HttpClient = HttpClient
module.exports.HttpError = require('./error/HttpError')
module.exports.RuntimeError = require('./error/RuntimeError')
module.exports.CrudController = require('./task/CrudController')