const http = require('http')
const https = require('https')
const url = require('url')
const HttpBody = require('./HttpBody')
const HttpForm = require('./HttpForm')
const HttpJson = require('./HttpJson')
const HTTP = /^https?:\/\//i

module.exports = class HttpClient {
  constructor(domain = '') {
    this._domain = domain
  }

  static isAbsoluteUrl(value) {
    return HTTP.test(value)
  }

  static json(value) {
    return new HttpJson(value)
  }

  static form(value) {
    return new HttpForm(value)
  }

  request(method, api, { headers, query, body }) {
    let content = ''
    if (query) {
      api += '?' + HttpForm.encode(query)
    }
    if (body) {
      let contentType = body.contentType
      content = body.toString()
      headers = Object.assign({
        'Content-Type': contentType,
        'Content-Length': Buffer.byteLength(content)
      }, headers)
    }
    if (!HttpClient.isAbsoluteUrl(api)) {
      api = this._domain + api
    }
    let options = url.parse(api)
    options.method = method
    options.headers = headers
    return new Promise((resolve, reject) => {
      let proxy = options.protocol === 'http:' ? http : https
      let req = proxy.request(options, res => {
        let buf = ''
        res.setEncoding('utf8')
        res.on('data', (chunk) => {
          buf += chunk
        })
        res.on('end', () => {
          resolve({
            status: res.statusCode,
            headers: res.headers,
            body: buf
          })
        })
      })
      req.on('error', err => {
        reject(err)
      })
      if (content) {
        req.write(content)
      }
      req.end()
    })
  }

  get(api, { headers, query } = {}) {
    return this.request('GET', api, { headers, query })
  }

  delete(api, { headers, query } = {}) {
    return this.request('DELETE', api, { headers, query })
  }

  post(api, { headers, query, body } = {}) {
    return this.request('POST', api, { headers, query, body })
  }

  put(api, { headers, query, body } = {}) {
    return this.request('PUT', api, { headers, query, body })
  }

  patch(api, { headers, query, body } = {}) {
    return this.request('PATCH', api, { headers, query, body })
  }
}
