const http = require('http')
const express = require('express')
const bodyParser = require('body-parser')
const cookieParser = require('cookie-parser')
const multer = require('multer')

module.exports = class HttpServer {
  constructor(port, staticRoot) {
    if (typeof port == 'number') {
      port = [port]
    }
    this._ports = port
    this._router = express.Router()
    this._handler = express()
    this._handler.use(bodyParser.json())
    this._handler.use(bodyParser.urlencoded({ extended: true }))
    this._handler.use(bodyParser.text())
    this._handler.use(cookieParser())
    this._handler.use(this._router)
    if (staticRoot) {
      this._handler.use('/static', express.static(staticRoot))
    }
    this._handler.use((err, req, res, next) => {
      console.error(err.stack)
      res.status(err.status || 500).send(err.message || 'Internal Server Error')
    })
  }

  addTask(method, url, task, uploadFields) {
    if (uploadFields) {
      if (!this._upload) {
        this._upload = multer({ dest: 'uploads/' })
      }
      this._router[method](url, this._upload.fields(uploadFields), task)
    } else {
      this._router[method](url, task)
    }
  }

  async startup() {
    console.info('try listen', this._ports)
    for (let i = 0; i < this._ports.length; i++) {
      try {
        this._server = await this.startListen(this._ports[i], this._handler)
        console.log('http server is startup on ' + this._ports[i])
        break;
      } catch (e) {
        console.error(e.message, 'try next ...')
      }
    }
  }

  async startListen(port, handler) {
    return new Promise(function (resolve, reject) {
      let server = http.Server(handler)
      server.on('error', (e) => {
        server.close()
        reject(e)
      })
      server.on('listening', () => {
        resolve(server)
      })
      server.listen(port)
    })
  }

  shutdown() {
    this._server.close()
  }
}
