module.exports = class CrudController {
    constructor(context, repository, handlers) {
        this._context = context
        this._repository = repository
        this._handlers = [...this.crudHandlers, ...handlers]
    }
    get context() {
        return this._context
    }

    get handlers() {
        return this._handlers
    }
    get crudHandlers() {
        return [
            {
                url: '/',
                method: 'post',
                task: this.create.bind(this)
            },
            {
                url: '/',
                method: 'get',
                task: this.retrievePage.bind(this)
            },
            {
                url: '/:id',
                method: 'get',
                task: this.retrieve.bind(this)
            },
            {
                url: '/:id',
                method: 'patch',
                task: this.update.bind(this)
            },
            {
                url: '/:id',
                method: 'delete',
                task: this.delete.bind(this)
            }
        ]
    }

    async create(context, entity) {
        if (!entity) {
            const body = context.request.body
            entity = {}
            for (let i = 0; i < this._repository.schema.fields.length; i++) {
                let field = this._repository.schema.fields[i]
                if (body[field.name] != undefined) {
                    entity[field.name] = body[field.name]
                }
            }
        }
        return await this._repository.insert(context, entity)
    }

    async retrievePage(context) {
        const { page, size } = context.request.query
        return await this._repository.select(context, false, { page, size })
    }

    async retrieve(context) {
        const { id } = context.request.params
        return await this._repository.get(context, id)
    }

    async update(context, entity) {
        if (!entity) {
            const { id } = context.request.params
            const body = context.request.body
            entity = {}
            for (let i = 0; i < this._repository.schema.fields.length; i++) {
                let field = this._repository.schema.fields[i]
                if (body[field.name] != undefined) {
                    entity[field.name] = body[field.name]
                }
            }
        }
        return await this._repository.update(context, entity)
    }

    async delete(context) {
        const { id } = context.request.params
        return await this._repository.delete(context, id)
    }
}