module.exports = class TaskContext {
  constructor(taskManager, task, isTransaction, prev) {
    this._taskManager = taskManager
    this._task = task
    this._isTransaction = isTransaction
    this._prev = prev
  }

  get taskManager() {
    return this._taskManager
  }

  async _loadSqlContext() {
    if (!this._sqlContext) {
      this._sqlContext = await this.taskManager.createSqlContext()
    }
  }

  async preRun() {
    if (this._isTransaction) {
      this._isTransaction = false
      await this.beginTransaction()
    }
  }

  postRun(result) {
    if (this._prev) {
      this._prev.onContinue(result)
    }
  }

  async run() {
    try {
      await this.preRun()
      let result = await this._task(this)
      await this.commit()
      if (!this._hangUp) {
        this.postRun(result)
      }
    } catch (err) {
      await this.rollback()
      throw err
    } finally {
      this.release()
    }
  }

  onContinue(result) {
    this._hangUp = false
    if (this._callbackTask) {
      this._task = this._callbackTask
      this._callbackTask = null
      this.taskManager.execute(this)
    } else {
      this.postRun(result)
    }
  }

  scheduleNext(task, callbackTask) {
    this._hangUp = true
    this._callbackTask = callbackTask
    this.taskManager.execute(new TaskContext(this.taskManager, task, false, this))
  }

  async query(sql, values) {
    await this._loadSqlContext()
    return await this._sqlContext.query(sql, values)
  }

  async beginTransaction() {
    if (!this._isTransaction) {
      await this._loadSqlContext()
      this._isTransaction = true
      return await this._sqlContext.beginTransaction()
    }
  }

  async commit() {
    if (this._isTransaction && this._sqlContext) {
      return await this._sqlContext.commit()
    }
  }

  async rollback() {
    if (this._isTransaction && this._sqlContext) {
      return await this._sqlContext.rollback()
    }
  }

  release() {
    if (this._sqlContext) {
      this._sqlContext.release()
      this._sqlContext = null
    }
  }
}
