module.exports = class HttpResponse {
  constructor(res) {
    this._res = res
    this._isSent = false
  }

  get isSent() {
    return this._isSent || this._res.headersSent
  }

  setHeader(name, value) {
    this._res.set(name, value)
  }

  setCookie(name, value) {
    this._res.cookie(name, value, { httpOnly: true })
  }

  removeCookie(name) {
    this._res.clearCookie(name)
  }

  redirect(url, status = 302) {
    this._isSent = true
    this._res.redirect(status, url)
  }

  send(obj, status = 200) {
    this._isSent = true
    this._res.status(status)
    obj ? this._res.send(obj) : this._res.end()
  }

  created(obj) {
    this._isSent = true
    this.send(obj, 201)
  }

  accepted() {
    this._isSent = true
    this.send(false, 202)
  }

  noContent() {
    this._isSent = true
    this.send(false, 204)
  }

  badRequest(message) {
    this._isSent = true
    this.send(message, 400)
  }

  unAuthorized(message) {
    this._isSent = true
    this.send(message, 401)
  }

  forbidden(message) {
    this._isSent = true
    this.send(message, 403)
  }

  notFound(message) {
    this._isSent = true
    this.send(message, 404)
  }

  sendFile(path, name) {
    this._isSent = true
    if (name) {
      this.setHeader('Content-Disposition', 'attachment filename=' + name)
    }
    this._res.sendFile(path, (err) => {
      if (err) {
        console.error(err)
      }
    })
  }
}
