const TaskContext = require('../TaskContext')
const HttpRequest = require('./HttpRequest')
const HttpResponse = require('./HttpResponse')
const HttpError = require('../../error/HttpError')
const Verify = require('../../util/Verify')
const ARGUMENT_FORMAT = '请求参数格式错误'

module.exports = class HttpTaskContext extends TaskContext {
  constructor(taskManager, handler, options) {
    super(taskManager, handler.task, options.isTransaction)
    this._handler = handler
    this._options = options
    this._reqeust = new HttpRequest(options.req)
    this._response = new HttpResponse(options.res)
  }

  get request() {
    return this._reqeust
  }

  get response() {
    return this._response
  }

  async preRun() {
    this.verifyQuery()
    this.verifyBody()
    this.verifyFiles()
    await super.preRun()
  }

  verifyRequest(source, target) {
    let result = {}
    for (let i = 0; i < target.length; i++) {
      let options = target[i]
      let key = options.name
      if (Reflect.has(source, key)) {
        result[key] = source[key]
        if (options.verify) {
          if (typeof options.verify === 'string') {
            if (Verify[options.verify]) {
              if (!Verify[options.verify].test(result[key])) {
                throw new HttpError(400, ARGUMENT_FORMAT)
              }
            } else {
              throw new HttpError(500, 'not found verify:' + options.verify)
            }
          } else if (typeof options.verify === 'function') {
            if (!options.verify(result[key])) {
              throw new HttpError(400, ARGUMENT_FORMAT)
            }
          } else if (options instanceof RegExp) {
            if (!options.verify.test(result[key])) {
              throw new HttpError(400, ARGUMENT_FORMAT)
            }
          }
        }
      } else {
        if (options.required) {
          throw new HttpError(400, ARGUMENT_FORMAT)
        }
      }
    }
    return result
  }

  verifyQuery() {
    if (this._handler.query) {
      this.request.query = this.verifyRequest(this.request.query, this._handler.query)
    }
  }

  verifyBody() {
    if (this._handler.body) {
      this.request.body = this.verifyRequest(this.request.body, this._handler.body)
    }
  }

  verifyFiles() {
    if (this._handler.files) {
      this.request.files = this.verifyRequest(this.request.files, this._handler.files)
    }
  }

  postRun(result) {
    if (!this.response.isSent) {
      this.response.setHeader('Cache-Control', 'no-cache')
      if (result === undefined) {
        this.response.notFound()
      } else {
        this.response.send(result)
      }
    }
  }

  async run(...args) {
    try {
      await super.run(...args)
    } catch (e) {
      this._options.next(e)
    }
  }
}
