import { JpaConfig, JpaContext, Repository } from 'js-jpa'

export function startup(runtime: string, config: AppConfig): Promise<void>
export function shutdown(): Promise<void>
export function scan(path: string, filter: ScanFilter): Promise<void>
export function installController(controller: Controller): void
export function installRepositories(path: string): Promise<void>
export function installRepository(context: JpaContext, repository: Repository): Promise<void>
export function repository<T>(repository: T): Repository & T
export function schedule(task: ContextTask, isTransaction: boolean)

export class RuntimeError {
    constructor(message: string, cause: Error)
    cause: Error
}
export class HttpError extends RuntimeError {
    constructor(status: number, message: string, cause?: Error)
    status: number
}

export class HttpClient {
    constructor(domain: string)
    request(method: string, api: string, { headers, query, body: HttpBody }): Promise<HttpResponse>
    get(api: string, { headers, query }): Promise<HttpResponse>
    delete(api: string, { headers, query }): Promise<HttpResponse>
    post(api: string, { headers, query, body: HttpBody }): Promise<HttpResponse>
    put(api: string, { headers, query, body: HttpBody }): Promise<HttpResponse>
    patch(api: string, { headers, query, body: HttpBody }): Promise<HttpResponse>
    static isAbsoluteUrl(value: string): boolean;
    static json(value: any): HttpBody
    static form(value: any): HttpBody
}
export interface HttpBody {
}
export interface HttpResponse {
    status: number
    headers: any
    body: string
}

export interface AppConfig {
    appName: string
    appPort: number
    database?: JpaConfig
}

export interface ScanFilter {
    (mod): void
}

export interface Controller {
    path: string
    handlers: ControllerHandler[]
}

export interface ControllerHandler {
    path: string
    method?: string
    transaction?: boolean,
    query?: Property[],
    body?: Property[],
    files?: Property[],
    task: ControllerTask
}

export interface Property {
    name: string,
    required?: boolean,
    verify?: string | RegExp | VerifyFunction
}

export interface VerifyFunction {
    (value): boolean
}

export interface ControllerTask {
    (context: HttpContext): void
}

export interface ContextTask {
    (context: TaskContext): void
}

export interface TaskContext extends JpaContext {
    scheduleNext(task: ContextTask, callbackTask: ContextTask): void
}

export interface HttpContext extends TaskContext {
    request: HttpContextRequest
    response: HttpContextResponse
}

export interface HttpContextRequest {
    originalUrl: string
    path: string
    params: any
    query: any
    body: any
    files: any
    cookies: any
    ip: string
    getHeader(name: string): string
}

export interface HttpContextResponse {
    isSent: boolean
    setHeader(name: string, value: string): void
    setCookie(name: string, value: string): void
    removeCookie(name: string): void
    redirect(url: string, status: number): void
    send(obj: any, status: number): void
    created(obj: any): void
    accepted(): void
    noContent(): void
    badRequest(message: string): void
    unAuthorized(message: string): void
    forbidden(message: string): void
    notFound(message: string): void
    sendFile(path: string, name: string): void
}